require 'csv'
require_relative 'player'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
  class Game
    attr_reader :title
  
    def initialize(title)
      @title = title
      @players = []
    end

    def high_score_entry(player)
      "#{player.name.ljust(20, '.')} #{player.score}"
    end
  
    def load_players(filename)
      CSV.foreach(filename) do |line|
        add_player(Player.from_csv(line))
      end
  #     File.readlines(filename).each do |line|
  #       add_player(Player.from_csv(line))
  #     end
    end
  
    def save_high_scores(to_file = "high_scores.txt")
      File.open(to_file, "w") do |file|
        file.puts "#{@title} High Scores:"
        @players.sort.each { |sp| file.puts high_score_entry(sp)}
      end
    end
  
    def add_player(player)
      @players << player
    end
  
    def total_points
      @players.reduce(0) { |sum, p| sum + p.points }
    end

    def play(rounds)
      puts "\nThere are #{@players.size} players in #{@title}:"
      puts @players

      treasures = TreasureTrove::TREASURES
      puts "\nThere are #{treasures.size} treasures to be found:"
      treasures.each { |treasure| puts "A #{treasure.name} is worth #{treasure.points} points" }

      1.upto(rounds) do |round|
        if block_given?
          break if yield
        end            
      
        puts "\nRound: #{round}:"
        @players.each do |player|
          GameTurn.take_turn(player)
          puts player
        end
      end
      print_stats
    end

    def print_stats
      strong, wimpy = @players.partition { |p| p.strong? }
    
      puts "\n#{@title} Statistics:"

      puts "\n#{strong.size} strong player#{strong.size == 1 ? '' : 's'}:"
      strong.each { |s| s.print_name_and_health }

      puts "\n#{wimpy.size} wimpy player#{wimpy.size == 1 ? '' : 's'}:"
      wimpy.each { |w| w.print_name_and_health }
    
      puts "\n#{@title} High Scores:"
      @players.sort.each { |sp| puts high_score_entry(sp)}
    
      @players.sort.each do |player| 
        puts "\n#{player.name}\`s point totals:"
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
        puts "#{player.points} grand total points"
      end
    
      puts "\n#{total_points} total points from treasures found"
    end
  end
end
