require_relative 'treasure_trove'
require_relative 'playable'

module StudioGame
  class Player
    include Playable
  
    attr_accessor :name, :health
  
    def initialize(name, health = 100)
      @name = name.capitalize
      @health = health
      @found_treasures = Hash.new(0)
    end

    def name=(new_name)
      @name = new_name.capitalize
    end

    def self.from_csv(csv_line)
  #     name, health = csv_line.split(',')
      Player.new(csv_line[0], Integer(csv_line[1]))
    end
  
    def to_s
      "I'm #{@name} with health = #{@health}, points = #{points} and a score of #{score}."
    end
  
    def <=>(player)
      player.score <=> self.score
    end
  
    def print_name_and_health
      puts "#{@name} (#{@health})"
    end
  
    def score
      @health + points  
    end
  
    def found_treasure(treasure)
      @found_treasures[treasure.name] += treasure.points
      puts "#{@name} found a #{treasure.name} worth #{treasure.points} points."
      puts "#{@name}\'s treasures: #{@found_treasures}"
    end
  
    def each_found_treasure
      @found_treasures.each do |name, points|
        yield Treasure.new(name, points)
      end
    end
  
    def points
      @found_treasures.values.reduce(0, :+)
    end
  end


  if __FILE__ == $0
    player1 = Player.new("moe")
    puts player1
    puts player1.health
  end
end
